// Copyright (c) 2012 Google Inc. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// This file exists to test that valid C files compile correctly.

void FunctionInCFile(void) {
}
