#!/bin/bash
# Copyright (c) 2011 Google Inc. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

set -e
touch "${BUILT_PRODUCTS_DIR}/static_touch"
