// Copyright (c) 2011 Google Inc. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

int SuperFly() {
  return 42;
}

const char* SuperFoo() {
  return "Hello World";
}
